<?PHP
/*
 *   CLIGS v 0.2 - Cute Little Image Gallery Script
 *   (c) Copyright 2005 John Leach <john@johnleach.co.uk> 
 *   http://johnleach.co.uk/documents/cligs
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
*/

/* if you can't put navigation images and stylesheets in the root of your
 * domain, specify the subfolders up to it here */
$base = "";

/* you can specify your own thumb cache directory if you want.  Relative to the
 * cligs.php script */
$thumb_dir = "thumbcache/";

include "HTML/Page2.php";

$docroot = $_SERVER["DOCUMENT_ROOT"];
/* nasty hack to parse encoded spaces, FIXME: what about other characters?  other languages! */
$uri = str_replace( "%20", " ", $_SERVER["REQUEST_URI"] );
$host = $_SERVER["HTTP_HOST"];
$script = $_SERVER["SCRIPT_FILENAME"];
$pi = pathinfo( $script );
$scriptdir = $pi['dirname'];
$thumbroot = $scriptdir . "/" . $thumb_dir;

/* Cache of parsed gallery.xml files */
$gal_info_cache = array();

function gameover( $msg ) {
	print "<h1>Something isn't right</h1>";
	print "<p>Something is missing or isn't set up correctly.</p>";
	print "<p>$msg</p>";
	exit;
}

if ( !function_exists( "ImageCreateTrueColor" ) 
		or !function_exists( "imagecreatefromjpeg" ) 
		or !function_exists( "imagecreatefrompng" ) ) {
	gameover( "Image creation functions not found.  Does PHP have GD support installed?" );
}

if ( !defined( "HTML_PAGE2_APPEND" ) ) {
	gameover( "HTML_Page2 Pear module not found.  Have you install Pear, and this module?" );
}

if ( !function_exists( "xml_parse_into_struct" ) ) {
	gameover( "XML parsing functions not found.  Sorry." );
}

if ( !is_dir( $thumbroot ) ) {
	gameover( "Thumbnail cache directory doesn't exist");
}

if ( !is_writable( $thumbroot ) ) {
	gameover( "Thumbnail cache directory isn't writable to by the web server");
}


## HTML output setup
$hp = new HTML_Page2();
$hp->addHeadLink( "$base/cligs.css", "stylesheet" );
$hp->setMetaData( "Generator", "CLIGS 0.2" );
$hp->setTitle( "image gallery" );

## Output last modified header using file modification time
function lastmod( $req_filename ) {
	header( "Last-Modified: " . date( "D, d M Y H:i:s", filemtime( $req_filename ) )  . " GMT" );
}

## Read jpeg from disk and send it to browser with appropriate headers
function out_jpeg($filename) {
	lastmod( $filename );
	header("Content-type: image/jpeg");
	return readfile($filename);
}


## Open, resize and output a jpeg
function resize( $req_filename, $w, $h, $q=95, $logo=false ) {
	global $docroot, $thumbroot;
	$thumbname = str_replace( $docroot, "", $req_filename );
	$pi = pathinfo( $thumbname );
	$thumbdir = $pi['dirname'];
	$thumbdir = $thumbroot . str_replace( "/", "_", $thumbdir );
	$thumbname = $thumbdir . "/" . $pi['basename']  . "-$w-$h-$q";

	if ( !is_dir( $thumbdir ) ) {
		if ( !mkdir( $thumbdir ) ) {
			gameover("Couldn't create thumbcache directory (have you considered all the SafeMode related stuff?)");
		}
	}

	# Check the cache
	if ( file_exists( $thumbname ) 
			and ( filemtime( $req_filename ) < filemtime( $thumbname ) ) 
			and ( filemtime( $logo ) < filemtime( $thumbname ) ) ) {
		if ( out_jpeg( $thumbname ) ) {
			return;
		}
	}

	# Not cached so generate a new jpeg
	$o = imagecreatefromjpeg( $req_filename );
	$ow = imageSX( $o );
	$oh = imageSY( $o );

	if ( $logo )
		$logopng = imagecreatefrompng( $logo );
	
	if ( $h == 0 )
		$h = ( ( $w / $ow ) * $oh );
	if ( $w == 0 ) {
		$w = ( ( $h / $oh ) * $ow );
	}

	$nw = ( ($h / $oh ) * $ow );
	$nh = ( ($w / $ow ) * $oh );

	if ( $nw > $w ) {
		$nh = $h;
	}
	if ($nh > $h) {
		$nw = $w;
	}

	# If aspect ratio isn't preserved, try to center the image
	$y = 0;
	if ( $nh > $h )
		$y = ( $nh - $h ) / 2;

	if ( ( $ow < $w ) and ( $oh < $h ) ) {
		$nw = $ow;
		$nh = $oh;
		$h = $oh;
		$w = $ow;
	}

	$resize = ImageCreateTrueColor( $nw, $nh );
	$thumb = ImageCreateTrueColor( $w, $h );

	imagecopyresampled( $resize, $o, 
				0, 0, 0, 0,
				$nw, $nh,
				$ow, $oh );
	imagedestroy( $o );
	imagecopy( $thumb, $resize,
			0, 0, 0, $y,
			$w, $h );

	if ( isset( $logopng ) ) {
		imagecopy( $thumb, $logopng, 0, 0, 0, $y, imageSX( $logopng ), imageSY( $logopng ) );
	}
	
	imagedestroy( $resize );

	# Save to cache
	imagejpeg( $thumb, $thumbname, $q );
	
	imagedestroy( $thumb );

	if ( !is_readable( $thumbname ) ) {
		print $thumbname;
		gameover("thumbnail cache isn't readable.  Have you considered all the SafeMode related stuff?");
	}
	# Output from cache
	out_jpeg( $thumbname );
}

# read and parse the gallery.xml (Used by read_gallery_info())
function read_gallery_xml( $xmldir ) {
	global $gal_info_cache;

	/* serve from cache if available */
	if ( array_key_exists( $xmldir, $gal_info_cache ) )
		return array_key_exists( $gal_info_cache[$xmldir] );
		
	$mygal = array();
	$pi = pathinfo($xmldir);
	$xmldir = $pi['dirname'];
	$fname = $xmldir . "/" . $pi['basename'];
	if ( is_dir( $fname ) )
		$xmldir = $fname;
	$xml_filename = $xmldir . "/gallery.xml";
	if ( !file_exists( $xml_filename ) )
		return $mygal;

	$xmldata = join( file( $xml_filename ) );
	$p = xml_parser_create();
	xml_parse_into_struct( $p, $xmldata, $vals, $index );
	xml_parser_free( $p );
	foreach ( $vals as $v ) {
		if ( array_key_exists( 'value', $v ) )
			$mygal[$v['tag']] = $v['value'];
		}
	$gal_info_cache[$xmldir] = $mygal;
	return $mygal;
}

## Read gallery.xml file and return an array of the info
## Merges all gallery.xmls from parent dirs
function read_gallery_info( $dir ) {
	$gal = array();
	$p = "";
	foreach ( split( "/", $dir ) as $d ) {
		$p = $p . $d . "/";
		$gal['TITLE'] = false;
		if ( is_file( $p . "/gallery.xml" ) ) {
			$g = read_gallery_xml( $p );
			$gal = array_merge( $gal, $g );
			$gal = array_merge( $gal );
		}
	}
	
	if ( !array_key_exists( 'LOGO', $gal ) )
		$gal['LOGO'] = false;
	
	if ( !array_key_exists( 'FOOTER', $gal ) )
		$gal['FOOTER'] = false;
	
	if ( !array_key_exists( 'ICON', $gal ) )
		$gal['ICON'] = false;
		
	return $gal;
}

## Read gthumb xml file and return array of the info
function read_image_info( $filename ) {
	$phinfo = array();
	$phinfo['PLACE'] = NULL;
	$phinfo['shorttimestampstring'] = "";
	$phinfo['timestampstring'] = "Unknown time";
	$pi = pathinfo( $filename );
	$dir = $pi['dirname'];
	$xml_filename = $pi['dirname'] . "/.comments/" . $pi['basename'] . ".xml";
	
	# Read gthumb meta data
	if ( is_file( $xml_filename ) and function_exists( "gzfile" ) ) {
		$xmldata = join( gzfile( $xml_filename ) );
		$p = xml_parser_create();
		xml_parse_into_struct( $p, $xmldata, $vals, $index );
		xml_parser_free( $p );
		foreach ( $vals as $v ) {
			if ( array_key_exists( 'value', $v ) )
				$phinfo[ $v['tag'] ] = $v['value'];
		}
	}
	
	# Read EXIF metadata
	if ( function_exists( "exif_read_data" ) ) {
		if ( $exif = exif_read_data( $filename, 'IFD0' ) ) {
			foreach ( $exif as $key => $val ) {
				$phinfo[ $key ] = $val;
			}
		}
	}

	# Convert avail time metadata into a nice string
	if ( array_key_exists( 'TIME', $phinfo ) ) {
		$phinfo['timestampstring'] = strftime( "%B %d %Y, %R ", $phinfo['TIME'] );
		$phinfo['shorttimestampstring'] = strftime( "%b %Y", $phinfo['TIME'] );
	} elseif ( array_key_exists( 'DateTime', $phinfo ) )
		$phinfo['timestampstring'] = $phinfo['DateTime'];
	
	return $phinfo;
}

/* find all readable images in the specified path, return an array */
function get_img_list( $path ) {
	if ( is_file( $path ) ) {
		$pi = pathinfo( $path );
		$path = $pi['dirname'];
	}
	$path = $path . "/";
	$list = array();
	if ( $dh = opendir( $path ) ) {
		while ( ( $file = readdir( $dh ) ) !== false) {
			$fn = $path . $file;
			if ( !eregi( "^.*(\.jpg|\.jpeg)$", $fn ) )
				continue;
			if ( !is_file( $fn ) )
				continue;
			$list[] = $file;
		}
		closedir( $dh );
	}
	return $list;
}

/* find all directories in the specified path, return an array */
function get_dir_list( $path ) {
	if ( is_file( $path ) ) {
		$pi = pathinfo( $path );
		$path = $pi['dirname'];
	}
	$path = $path . "/";
	$list = array();
	if ( $dh = opendir( $path ) ) {
		while ( ( $file = readdir( $dh ) ) !== false ) {
			$fn = $path . $file;
			if ( !is_dir( $fn ) )
				continue;
			if ( eregi( "/\.+", $fn ) )
				continue;
			$list[] = $file;
		}
		closedir( $dh );
	}
	return $list;
}

/* add info page icon image list to html page output */
function draw_header_list() {
	global $hp, $loc_path, $loc_filepath, $uri, $req_path, $req_filename, $base;
	$dirlist = get_img_list( $loc_path );
	$hp->addbodycontent("<div id='header_list'>");
	$hp->addbodycontent("<a href='$req_path'><img src='$base/up.png' title='back to gallery' alt='up arrow'/></a>");
	foreach ( $dirlist as $file ) {
		$fn = $loc_path . $file;
		$rfp = $req_path . $file;
		$class = 'iconfade';
		if ( $req_filename == $file )
			$class ='icon';
		$phinfo = read_image_info( $fn );
		$hp->addbodycontent(
			sprintf( "<a href='%s:::info'><img class='%s' src='%s:::icon' title='%s' alt='thumbnail'/></a>",
					$rfp, $class, $rfp, $phinfo['PLACE'] ) );
	}
	$hp->addbodycontent("</div>");
}

/* add list of subdirectories to html page output */
function draw_gallery_list( $thisdir ) {
	global $hp;
	$dirlist = get_dir_list( $thisdir );
	if ( count( $dirlist ) > 0 ) {
		$hp->addbodycontent( "<div id='gallerylist'>" );
		$hp->addbodycontent( "<h1>Other galleries here:</h1>" );
		$hp->addbodycontent( "<ul>" );
		foreach ( $dirlist as $dir ) {
			unset( $thisgal );
			$thisgal = read_gallery_info( $thisdir . "/" . $dir );
			$title = $dir;
			if ( $thisgal['TITLE'] )
				$title = $thisgal['TITLE'];
			$hp->addbodycontent("<li><a href='$dir'><img src='$dir/:::icon' alt='icon' title='$title'/> $title</a></li>");
		}
		$hp->addbodycontent( "</ul>" );
		$hp->addbodycontent( "</div>" );
	}
	if ( count( $dirlist ) == 1 )
		return $dirlist[0];
	else
		return False;
}


## Parse the request url
$l = split( ":::", $uri );
if ( count( $l ) < 2 ) {
	$l[0] = $uri;
	$l[1] = false;
}

$pi = pathinfo( $l[0] );
$req_filename = $pi['basename'];
$req_path = $pi['dirname'] . "/";
$req_filepath = $req_path . $req_filename;
$loc_path = $docroot . $req_path;
$loc_filepath = $loc_path . $req_filename;
$mode = $l[1];


# Output a thumbnail sized jpeg (160 x 120)
if ($mode == "thumb") {
	resize( $loc_filepath, 160, 120, 65 );
	return 1;
};

# Output a preview sized jpeg (450 x ??)
if ($mode == "preview") {
	$gal = read_gallery_info( $loc_filepath );
	resize( $loc_filepath, 450, 0, 80, $gal["LOGO"] );
	return 1;
};

# Output an icon sized jpeg (45 x 35)
if ($mode == "icon") {
	if ( is_dir( $loc_filepath ) ) {
		/* use the specified icon is possible */
		$gal = read_gallery_info( $loc_filepath );
		if ( array_key_exists( 'ICON', $gal ) and is_file( $loc_filepath . '/' . $gal['ICON'] ) ) {
			$loc_filepath = $loc_filepath . '/' . $gal['ICON'];
		} else {
			/* choose a random image from the dir */
			$fl = get_img_list( $loc_filepath );
			$loc_filepath = $loc_filepath . "/" . $fl[rand( 0, count( $fl ) - 1 )];
		}
	}
	
	resize( $loc_filepath, 45, 35, 75 );
	return 1;
};

# Output an information page for the selected image
if ($mode == "info") {
	lastmod( $loc_filepath );
	$gal = read_gallery_info( $loc_filepath );
	$phinfo = read_image_info( $loc_filepath );
	$hp->settitle( $gal['TITLE'] . " : " . $phinfo['PLACE'] . " : " . $phinfo['shorttimestampstring'] );
	draw_header_list( $loc_path );
	$hp->addbodycontent( sprintf( "<h1>%s</h1>", $gal['TITLE'] ) );
	$hp->addbodycontent( "<div id='infobox'>" );
	$hp->addbodycontent( sprintf( "<a href='%s'><img src='%s:::preview' alt='preview'/></a>",
				$req_filepath, $req_filepath ) );
	$hp->addbodycontent( "<div id='info'>" );
	$hp->addbodycontent( sprintf( "<h1>%s</h1>", $phinfo['PLACE'] ) );
	$hp->addbodycontent( sprintf("<h2>%s</h2>", $phinfo['timestampstring'] ) );
	$hp->addbodycontent( sprintf("<p>%s</p>", $phinfo['NOTE'] ) );
	$hp->addbodycontent( "</div></div>" );
	$hp->addbodycontent( "<div id='footer'><p>" . $gal['FOOTER'] . "</p><p>Gallery created using <a href='http://johnleach.co.uk/documents/cligs/'>CLIGS</a></p></div>" );
	$hp->display();
	return 1;
}

/* Output the full image with no resizing and no overlays */
if ($mode == "full") {
	out_jpeg( $loc_filepath );
	return 1;
}


# Output a large size image
if (is_file( $loc_filepath ) ) {
	$gal = read_gallery_info( $loc_filepath );
	resize( $loc_filepath, 1024, 0, 95, $gal['LOGO'] );
	return 1;
}

# Output a gallery list of the selected directory
if ( is_dir( $loc_filepath ) ) {
	# Redirect to a trailing slash if necessary
	if ( $uri[ strlen( $uri ) - 1 ] != '/' ) {
		header( "Location: ". $uri . "/" );
		return;
	}

	lastmod( $loc_filepath );

	$gal = read_gallery_info( $loc_filepath );

	$title = $gal['TITLE'];

	if ( !$title )
		$title = str_replace( $scriptdir, "", $loc_filepath );
	$hp->settitle( $title );

	/* draw the header */
	$hp->addbodycontent( "<div id='header'>" );
	$hp->addbodycontent( "<a href='../'><img src='$base/up.png' alt='up arrow'/></a><h1>$title</h1>" );
	if ( isset( $gal['DESCRIPTION'] ) )
		$hp->addbodycontent( sprintf( "<p>%s</p>", $gal['DESCRIPTION'] ) );
	$hp->addbodycontent( "</div>" );

	/* add links to any other sub directories */
	$subdir = draw_gallery_list( $loc_filepath );

	/* add the thumbnails */
	$hp->addbodycontent( "<div id='imagelist'>" );
	$imglist = get_img_list( $loc_filepath );
	foreach ( $imglist as $file ) {
		$fn = $loc_filepath . "/" . $file;
		$hp->addbodycontent( "<div class='thumbbox'>" );
		$hp->addbodycontent( sprintf( "<a href='%s:::info'><img src='%s:::thumb' title='%s' alt=''/></a>",
					$uri . $file, $uri . $file, $file ) );
		$phinfo = read_image_info( $fn );
		$hp->addbodycontent( sprintf( "<h1>%s</h1>", $phinfo['PLACE'] ) );
		$hp->addbodycontent( sprintf( "<h2>%s</h2>",$phinfo['shorttimestampstring'] ) );
		$hp->addbodycontent( "</div>" );
	}
	$hp->addbodycontent( "</div>" );

	/* If no images in this dir and one sub directory exists, jump to it straight away */
	if ( $subdir && ( count( $imglist ) == 0 ) ) {
		header( "Location: " . $uri . $subdir );
		return;
	}

	/* Draw the footer */
	$hp->addbodycontent( "<br clear='all'/><br/>" );
	$hp->addbodycontent( "<div id='footer'><p>" . $gal['FOOTER'] . "</p><p>Gallery created using <a href='http://johnleach.co.uk/documents/cligs/'>CLIGS</a></p></div>" );

	$hp->display();
}

?>
